--[[ 
SwitchSpecs 7.3.0.01

SwitchSpecs is a simple addon that allows the user to change between specs and
gear sets with a single button.

Users will need to label thier gear sets according to their respective 
specializations.

This version requires the user to write a short macro (see below) and requires a
mouse with three buttons to access all specializations.

Required Macro
/switchspecs
--]]

local addonName, addonTable = ...
local SwitchSpecsFrame								-- Frame that listens for ADDON_LOADED event
-- local SwitchSpecs = {}
addonTable.Version = "7.3.0.01"
addonTable.Author = "TheRealChappy"


-- Onload initializes Let's Ride after the ADDON_LOADED event occurs
local function OnLoad(self, event, myName, ...)
	if myName == "SwitchSpecs" then
  
      SLASH_SWITCHSPECS1, SLASH_SWITCHSPECS2 = '/ss', '/switchspecs';	-- Simple slash command to access addon
				
	  addonTable.MessageOut("v"..addonTable.Version.." "..addonTable.loaded);
	end
  
end
-- Simple slash command to access addon
-- SLASH_SWITCHSPECS1, SLASH_SWITCHSPECS2 = '/ss', '/switchspecs';


-- MessageOut handles printing messages to the user
function addonTable.MessageOut(msg, r, g, b)
	if not r then r = 1; end
	if not g then g = 1; end
	if not b then b = 1; end

  msg = ("|cFF0070DESwitch Specs:|r "..msg);
  DEFAULT_CHAT_FRAME:AddMessage(msg, r, g, b);
  
end


-- checkGear() takes a string = EquipmentSet and tries to equip it
-- This is the second and final attempt to equip an item set.
-- If the equipment set cannot be equipped allow the default error
-- warn users that the equipment cannot be equipped.
function addonTable.checkGear(newSet)
	if type(newSet)~="number" then
	  return false;
	end
  C_EquipmentSet.UseEquipmentSet(newSet);
 
end

 
-- Values for _wait()
local waitTable = {};
local waitFrame = nil;

-- _wait() takes a number and a function with arguments
-- code was provided by WoWWiki wowwiki.wikia.com/wiki/Wait
function addonTable.wait(delay, func, ...)
	if(type(delay)~="number" or type(func)~="function") then
      return false;
	end
	if(waitFrame == nil) then
      waitFrame = CreateFrame("Frame","WaitFrame", UIParent);
      waitFrame:SetScript("onUpdate",function (self,elapse)
		  local count = #waitTable;
		  local i = 1;
			while(i<=count) do
			  local waitRecord = tremove(waitTable,i);
			  local d = tremove(waitRecord,1);
			  local f = tremove(waitRecord,1);
			  local p = tremove(waitRecord,1);
				if(d>elapse) then
				  tinsert(waitTable,i,{d-elapse,f,p});
				  i = i + 1;
				else
				  count = count - 1;
				  f(unpack(p));
				end
			end
		end);
	end
  tinsert(waitTable,{delay,func,{...}});
  return true;
end


-- equipSet() takes a number = EquipmentSetID and tries to equip it
-- If unable to equip, wait until character has changed specializations
-- notify user.
-- Currently this does not suppress the error message
local equipFrame = nil;

function addonTable.equipSet(newSet)
	if type(newSet)~="number" then
	  return false;
	end
	
  local setName = C_EquipmentSet.GetEquipmentSetInfo(newSet);
  local _,currentSpec = GetSpecializationInfo(GetSpecialization());
  
  C_EquipmentSet.UseEquipmentSet(newSet);										-- Try to equip the specified EquipmentSet
 
	if equipFrame == nil then
	  equipFrame = CreateFrame("Frame", "MyEventFrame");		-- make a frame and call it MyEventFrame
	end
	
  equipFrame:RegisterEvent("EQUIPMENT_SWAP_FINISHED");			-- register the frame for event
  equipFrame:SetScript("OnEvent",function (self, event, ...)
	  local didSwitch, setName = ...;									-- Collect arguments
		if not didSwitch then											-- If failed to equipped Set
--		  addonTable.wait(5.6, addonTable.checkGear, SwitchSpec, newSet);				-- Try again after specialization switch complete
		  local _,name=GetSpecializationInfo(GetSpecialization()); 
		  addonTable.MessageOut(addonTable.failed .. setName .. addonTable.failed2 .. currentSpec .. addonTable.failed3)
		end
		newSet = nil;
	end )
 
end


function SlashCmdList.SWITCHSPECS(msg, editbox)
  
  local btn = GetMouseButtonClicked();
  local equipSetIDs = {};
  local specNames = {};
	
	for i=1, GetNumSpecializations() do 					-- Get Specialization names
	  local _,name=GetSpecializationInfo(i); 
	  specNames[i] = name;
	  equipSetIDs[i] = C_EquipmentSet.GetEquipmentSetID(name);
	end
	
	for i=GetNumSpecializations()+1, 6 do					-- Create places for alternate equipment sets
	  specNames[i] = "Alt"..i-GetNumSpecializations();
	  equipSetIDs[i] = C_EquipmentSet.GetEquipmentSetID(specNames[i]);
--	  print(equipSetIDs[i]);
	end

-- Hack solution for not having your weapon equipped when entering combat.
-- Better solution required in future update.	
	if InCombatLockdown() then 								-- If in combat
	  addonTable.MessageOut(addonTable.combat);					-- Warn the user
	  local _,currentSpec = GetSpecializationInfo(GetSpecialization());
	  local equipSetID = C_EquipmentSet.GetEquipmentSetID(currentSpec)
	  
		if (equipSetID == nil) then									-- Check if the Equipment Set has been created
	      addonTable.MessageOut(addonTable.eSet..currentSpec..addonTable.missing);
		else
		  local itemsIDs = C_EquipmentSet.GetItemIDs(equipSetID);		-- Collect the items on the equipment set
			if (not IsEquippedItem(itemsIDs[INVSLOT_MAINHAND])) then
				if (itemsIDs[INVSLOT_OFFHAND] ~= nil) then
				  EquipItemByName(itemsIDs[INVSLOT_OFFHAND]);				-- Put the off hand weapon on the cursor
				else
				  EquipItemByName(itemsIDs[INVSLOT_MAINHAND]);				-- Put the main hand weapon on the cursor
				end
			end
		end
	
	elseif (IsShiftKeyDown() and btn == "MiddleButton") then	-- Else if user selected sixth spec function
		if (equipSetIDs[6] == nil) then								-- Check if the Equipment Set has been created
		  addonTable.MessageOut(addonTable.eSet..specNames[6]..addonTable.missing);
		else
		  addonTable.equipSet(equipSetIDs[6]);							-- Attempt to equip item set
		end
		
		if (GetSpecialization() ~=6 and GetNumSpecializations() >5) then	-- If user is not in sixth spec (currently not applicable)
		  SetSpecialization(6);										-- Switch to sixth spec
			if (equipSetIDs[6] ~= nil) then								-- If an associated equipment set exists
			  addonTable.wait(5.6, addonTable.checkGear, equipSetIDs[6]);		-- Verify it is equiped.
			end
		end
	
	elseif (IsShiftKeyDown() and btn == "RightButton") then	-- Else if user selected fifth spec function
		if (equipSetIDs[5] == nil) then							-- Check if the Equipment Set has been created
		  addonTable.MessageOut(addonTable.eSet..specNames[5]..addonTable.missing);
		else
		  addonTable.equipSet(equipSetIDs[5]);							-- Attempt to equip item set
		end
		
		if (GetSpecialization() ~=5 and GetNumSpecializations() >4) then	-- If user is not in fifth spec (currently not applicable)
		  SetSpecialization(5);										-- Switch to fifth spec
			if (equipSetIDs[5] ~= nil) then								-- If an associated equipment set exists
			  addonTable.wait(5.6, addonTable.checkGear, equipSetIDs[5]);		-- Verify it is equiped.
			end
		end
	
	elseif (IsShiftKeyDown() and btn == "LeftButton") then	-- Else if user selected fourth spec function
		if (equipSetIDs[4] == nil) then							-- Check if the Equipment Set has been created
		  addonTable.MessageOut(addonTable.eSet..specNames[4]..addonTable.missing);
		else
		  addonTable.equipSet(equipSetIDs[4]);							-- Attempt to equip item set
		end
		
		if (GetSpecialization() ~=4 and GetNumSpecializations() >3) then						-- If user is not in forth spec
		  SetSpecialization(4);										-- Switch to forth spec
			if (equipSetIDs[4] ~= nil) then								-- If an associated equipment set exists
			  addonTable.wait(5.6, addonTable.checkGear, equipSetIDs[4]);		-- Verify it is equiped.
			end
		end
	
	elseif (btn == "MiddleButton") then						-- Else if user selected third spec function
		if (equipSetIDs[3] == nil) then							-- Check if the Equipment Set has been created
		  addonTable.MessageOut(addonTable.eSet..specNames[3]..addonTable.missing);
		else
		  addonTable.equipSet(equipSetIDs[3]);							-- Attempt to equip item set
		end
		
		if (GetSpecialization() ~=3 and GetNumSpecializations() >2) then	-- If user is not in third spec
		  SetSpecialization(3);										-- Switch to third spec
			if (equipSetIDs[3] ~= nil) then								-- If an associated equipment set exists
			  addonTable.wait(5.6, addonTable.checkGear, equipSetIDs[3]);		-- Verify it is equiped.
			end
		end
	
	elseif (btn == "RightButton") then						-- Else if user selected second spec function
		if (equipSetIDs[2] == nil) then							-- Check if the Equipment Set has been created
		  addonTable.MessageOut(addonTable.eSet..specNames[2]..addonTable.missing);
		else
		  addonTable.equipSet(equipSetIDs[2]);								-- Attempt to equip item set
		end
		
		if (GetSpecialization() ~=2 and GetNumSpecializations() >1) then	-- If user is not in second spec
		  SetSpecialization(2);										-- Switch to second spec
			if (equipSetIDs[2] ~= nil) then								-- If an associated equipment set exists
			  addonTable.wait(5.6, addonTable.checkGear, equipSetIDs[2]);		-- Verify it is equiped.
			end
		end
	
	elseif (btn == "LeftButton") then						-- Else if user selected first spec function
		if (equipSetIDs[1] == nil) then						-- Check if the Equipment Set has been created
		  addonTable.MessageOut(addonTable.eSet..specNames[1]..addonTable.missing);
		else
		  addonTable.equipSet(equipSetIDs[1]);						-- Attempt to equip item set
		end
		
		if (GetSpecialization() ~=1) then						-- If user is not in first spec
		  SetSpecialization(1);										-- Switch to first spec
			if (equipSetIDs[1] ~= nil) then								-- If an associated equipment set exists
			  addonTable.wait(5.6, addonTable.checkGear, equipSetIDs[1]);		-- Verify it is equiped.
			end
		end
			
	end
	
end

--Initialize()
SwitchSpecsFrame = CreateFrame('Frame')
SwitchSpecsFrame:SetScript('OnEvent', OnLoad);
SwitchSpecsFrame:RegisterEvent('ADDON_LOADED');